package main

import (
	"errors"
	"fmt"
	"github.com/gofiber/fiber/v2"
	"github.com/gofiber/template/html/v2"
	"log"
)

const (
	ADDRESS = ":8000"
)

var (
	commands = map[string]Command{
		"a": Command{Command: "aaa", Status: ExpectReturn("aaa")},  // work okay
		"b": Command{Command: "bbb", Status: ExpectReturn("bbb")},  // work okay
		"c": Command{Command: "bbb", Status: ExpectReturn("okay")}, // this function shall intentionally return an error
		"d": Command{Command: "zzz", Status: ErrorReturn},          // this function shall intentionally return an error
	}
)

func main() {
	engine := html.New("templates", ".html")                            // create template engine
	app := fiber.New(fiber.Config{Views: engine, ViewsLayout: "_base"}) // create web app with template engine
	app.Get("/", GetIndex)                                              // configure some URL and method handlers in the web app
	app.Get("/:command", GetCommand)
	log.Fatal(app.Listen(ADDRESS)) // run the web app
}

// show the templates/index.html template with some variables
func GetIndex(c *fiber.Ctx) error {
	return c.Render("index", fiber.Map{"commands": commands, "lastcmd": "", "value": ""})
}

// read "command" parameter, select the command from out commands map and run it
func GetCommand(c *fiber.Ctx) error {
	cmd := c.Params("command")
	command, ok := commands[cmd]
	if !ok {
		// we could also render a "templates/error.html" template here instead
		// of returning the error in JSON format,see below...
		return c.Status(fiber.StatusInternalServerError).JSON(fiber.Map{
			"error": true, "msg": fmt.Sprintf("Command %s not found", cmd),
		})
	}
	ret, err := command.Status(command.Command)
	if err != nil {
		// here we return the error with a template, see above...
		return c.Status(fiber.StatusInternalServerError).Render("error", fiber.Map{
			"error": true, "msg": err.Error(),
		})
		// return c.Status(fiber.StatusInternalServerError).JSON(fiber.Map{
		// 	"error": true, "msg": err.Error(),
		// })
	}
	log.Printf("Command %s executed, it returned \"%s\"", cmd, ret)
	return c.Render("index", fiber.Map{"commands": commands, "lastcmd": cmd, "value": ret})
}

// just a simple type for our status function(s)
type StatusFunc func(value string) (string, error)

// the data structure for our commands map values
type Command struct {
	Command string
	Status  StatusFunc
}

// this is a wrapper so we can configure whats expected from the other side
func ExpectReturn(expected string) StatusFunc {
	return func(value string) (string, error) {
		if value == expected {
			return fmt.Sprintf("value was %s as expected", expected), nil
		}
		return "", errors.New("the returned value was not \"ok\"")
	}
}

// little helper that always fails, just as a demo
func ErrorReturn(value string) (string, error) {
	return "", errors.New("This function always returns an error.")
}
